// ExtractCallbackConsole.h

#include "StdAfx.h"

#include "Dialog.h"											// ǉ
#include "resource.h"										// ǉ

#include "../../UI/Console/ExtractCallbackConsole.h"		//	pXύX
//#include "UserInputUtils.h"								//	폜
#include "../../UI/Console/ConsoleClose.h"					//	pXύX

#include "Common/Wildcard.h"

#include "Windows/FileDir.h"
#include "Windows/FileFind.h"
#include "Windows/Time.h"
#include "Windows/Defs.h"
#include "Windows/PropVariant.h"
#include "Windows/Error.h"
#include "Windows/PropVariantConversions.h"

#include "../../Common/FilePathAutoRename.h"

#include "../../UI/Common/ExtractingFilePath.h"				//	pXύX

using namespace NWindows;
using namespace NFile;
using namespace NDirectory;

static const char *kTestingString    =  "Testing     ";
static const char *kExtractingString =  "Extracting  ";
static const char *kSkippingString   =  "Skipping    ";

//static const char *kCantAutoRename = "can not create file with auto name\n";
//static const char *kCantRenameFile = "can not rename existing file\n";
//static const char *kCantDeleteOutputFile = "can not delete output file ";
static const char *kError = "ERROR: ";
static const char *kMemoryExceptionMessage = "Can't allocate required memory!";

static const char *kProcessing = "Processing archive: ";
static const char *kEverythingIsOk = "Everything is Ok";
static const char *kNoFiles = "No files to process";

static const char *kUnsupportedMethod = "Unsupported Method";
static const char *kCrcFailed = "CRC Failed";
static const char *kCrcFailedEncrypted = "CRC Failed in encrypted file. Wrong password?";
static const char *kDataError = "Data Error";
static const char *kDataErrorEncrypted = "Data Error in encrypted file. Wrong password?";
static const char *kUnknownError = "Unknown Error";

STDMETHODIMP CExtractCallbackConsole::SetTotal(UInt64 size)		// ύX
{
  if (g_StdOut.GetProgressDialog()) g_StdOut.GetProgressDialog()->SetTotal(size);					// ǉ
  if (NConsoleClose::TestBreakSignal())
    return E_ABORT;
  return S_OK;
}

STDMETHODIMP CExtractCallbackConsole::SetCompleted(const UInt64 *completeValue)		// ύX
{
  if (NConsoleClose::TestBreakSignal())
    return E_ABORT;

  if (completeValue && g_StdOut.GetProgressDialog())				// ǉ
    g_StdOut.GetProgressDialog()->SetCompleted(*completeValue);		// ǉ

  return S_OK;
}

STDMETHODIMP CExtractCallbackConsole::AskOverwrite(
    const wchar_t *existName, const FILETIME *existTime, const UInt64 *existSize,	// ύX
    const wchar_t *newName, const FILETIME *newTime, const UInt64 *newSize,			// ύX
    Int32 *answer)
{
//  (*OutStream) << "file " << existName <<										// 폜
//    "\nalready exists. Overwrite with " << endl;								// 폜
//  (*OutStream) << newName;													// 폜
  
//  NUserAnswerMode::EEnum overwriteAnswer = ScanUserYesNoAllQuit(OutStream);	// 폜
  if (g_StdOut.GetProgressDialog() == NULL)
    return S_OK;
  CConfirmationDialog dlg;
  dlg.SetFileInfo(newName, *newSize, *newTime, existName, *existSize, *existTime);
  switch(dlg.DoModal())
  {
    case IDCANCEL:								// ύX
      return E_ABORT;
    case IDB_NO:
      *answer = NOverwriteAnswer::kNo;			// ύX
      break;
    case IDB_NO_ALL:
      *answer = NOverwriteAnswer::kNoToAll;		// ύX
      break;
    case IDB_YES_ALL:
      *answer = NOverwriteAnswer::kYesToAll;	// ύX
      break;
    case IDB_YES:
      *answer = NOverwriteAnswer::kYes;			// ύX
      break;
    case IDB_AUTO_RENAME:
      *answer = NOverwriteAnswer::kAutoRename;	// ύX
      break;
    default:
      return E_FAIL;
  }
  return S_OK;
}

STDMETHODIMP CExtractCallbackConsole::PrepareOperation(const wchar_t *name, bool /* isFolder */, Int32 askExtractMode, const UInt64 *position)
{
  switch (askExtractMode)
  {
    case NArchive::NExtract::NAskMode::kExtract:
      (*OutStream) << kExtractingString;
      break;
    case NArchive::NExtract::NAskMode::kTest:
      (*OutStream) << kTestingString;
      break;
    case NArchive::NExtract::NAskMode::kSkip:
      (*OutStream) << kSkippingString;
      break;
  };
  (*OutStream) << name;
  if (position != 0)
    (*OutStream) << " <" << *position << ">";
  return S_OK;
}

STDMETHODIMP CExtractCallbackConsole::MessageError(const wchar_t *message)
{
  (*OutStream) << message << endl;
  NumFileErrorsInCurrentArchive++;
  NumFileErrors++;
  return S_OK;
}

STDMETHODIMP CExtractCallbackConsole::SetOperationResult(Int32 operationResult, bool encrypted)
{
  switch(operationResult)
  {
    case NArchive::NExtract::NOperationResult::kOK:
      break;
    default:
    {
      NumFileErrorsInCurrentArchive++;
      NumFileErrors++;
      (*OutStream) << "     ";
      switch(operationResult)
      {
        case NArchive::NExtract::NOperationResult::kUnSupportedMethod:
          (*OutStream) << kUnsupportedMethod;
          break;
        case NArchive::NExtract::NOperationResult::kCRCError:
          (*OutStream) << (encrypted ? kCrcFailedEncrypted: kCrcFailed);
          break;
        case NArchive::NExtract::NOperationResult::kDataError:
          (*OutStream) << (encrypted ? kDataErrorEncrypted : kDataError);
          break;
        default:
          (*OutStream) << kUnknownError;
      }      
      /* ǉ */
      if (encrypted)
      {
        (*OutStream) << endl;
		DeleteFileAlways(g_StdOut.GetProgressDialog()->GetDestFilePath());
        return ERROR_PASSWORD_FILE;
      }
      /* ǉ܂ */
    }
  }
  (*OutStream) << endl;
  return S_OK;
}

STDMETHODIMP CExtractCallbackConsole::CryptoGetTextPassword(BSTR *password)
{
  if (!PasswordIsDefined)
  {
//    Password = GetPassword(OutStream);	// 폜
	/* ǉ */
    CPasswordDialog dlg(FALSE);
    dlg.DoModal();
    Password = dlg.GetPassword();
	/* ǉ܂ */
    PasswordIsDefined = true;
  }
  CMyComBSTR tempName(Password);
  *password = tempName.Detach();
  return S_OK;
}

HRESULT CExtractCallbackConsole::BeforeOpen(const wchar_t *name)
{
  NumArchives++;
  NumFileErrorsInCurrentArchive = 0;
  (*OutStream) << endl << kProcessing << name << endl;
  if (g_StdOut.GetProgressDialog()) g_StdOut.GetProgressDialog()->SetArchiveFile(name);	// ǉ
  return S_OK;
}

HRESULT CExtractCallbackConsole::OpenResult(const wchar_t * /* name */, HRESULT result, bool encrypted)
{
  (*OutStream) << endl;
  if (result != S_OK)
  {
    (*OutStream) << "Error: ";
    if (encrypted)
      (*OutStream) << "Can not open encrypted archive. Wrong password?";
    else
      (*OutStream) << "Can not open file as archive";
    (*OutStream) << endl;
	if (encrypted)							// ǉ
		return ERROR_PASSWORD_FILE;			// ǉ
    NumArchiveErrors++;
  }
  return S_OK;
}
  
HRESULT CExtractCallbackConsole::ThereAreNoFiles()
{
  (*OutStream) << endl << kNoFiles << endl;
  return S_OK;
}

HRESULT CExtractCallbackConsole::ExtractResult(HRESULT result)
{
  if (g_StdOut.GetProgressDialog()) g_StdOut.GetProgressDialog()->SendExtractingInfo(ARCEXTRACT_END);		// ǉ
  if (result == S_OK)
  {
    (*OutStream) << endl;
    if (NumFileErrorsInCurrentArchive == 0)
      (*OutStream) << kEverythingIsOk << endl;
    else 
    {
      NumArchiveErrors++;
      (*OutStream) << "Sub items Errors: " << NumFileErrorsInCurrentArchive << endl;
    }
  }
  if (result == S_OK)
    return result;
//  NumArchiveErrors++;										// 폜
//  if (result == E_ABORT || result == ERROR_DISK_FULL)		// 폜
//    return result;										// 폜
  /* ǉ */
  if (result == E_ABORT || result == ERROR_PASSWORD_FILE)
  {
    NumArchiveErrors = 0;
	NumFileErrors = 0;
    return result;
  }
  NumArchiveErrors++;
  if (result == ERROR_DISK_FULL)
	  return ERROR_DISK_FULL;
  /* ǉ܂ */
  (*OutStream) << endl << kError;
  if (result == E_OUTOFMEMORY)
    (*OutStream) << kMemoryExceptionMessage;
  else
  {
    UString message;
    NError::MyFormatMessage(result, message);
    (*OutStream) << message;
  }
  (*OutStream) << endl;

  return S_OK;
}

HRESULT CExtractCallbackConsole::SetPassword(const UString &password)
{
  PasswordIsDefined = true;
  Password = password;
  return S_OK;
}
